@@ -6,7 +6,7 @@
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
  *
- *      http://www.apache.org/licenses/LICENSE-2.0
+ *      https://www.apache.org/licenses/LICENSE-2.0
  *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
@@ -35,7 +35,10 @@ import org.apache.commons.codec.DecoderException;
 import org.apache.commons.lang3.ArrayUtils;
 import org.junit.jupiter.api.Test;
 
-public class Base32Test {
+/**
+ * Tests {@link Base32}.
+ */
+class Base32Test {
 
     private static final Charset CHARSET_UTF8 = StandardCharsets.UTF_8;
 
@@ -157,7 +160,7 @@ public class Base32Test {
     // @formatter:on
 
     /**
-     * Test base 32 decoding of the final trailing bits. Trailing encoded bytes
+     * Test Base32 decoding of the final trailing bits. Trailing encoded bytes
      * cannot fit exactly into 5-bit characters so the last character has a limited
      * alphabet where the final bits are zero. This asserts that illegal final
      * characters throw an exception when decoding.
@@ -210,22 +213,22 @@ public class Base32Test {
     }
 
     @Test
-    public void testBase32AtBufferEnd() {
+    void testBase32AtBufferEnd() {
         testBase32InBuffer(100, 0);
     }
 
     @Test
-    public void testBase32AtBufferMiddle() {
+    void testBase32AtBufferMiddle() {
         testBase32InBuffer(100, 100);
     }
 
     @Test
-    public void testBase32AtBufferStart() {
+    void testBase32AtBufferStart() {
         testBase32InBuffer(0, 100);
     }
 
     @Test
-    public void testBase32BinarySamples() throws Exception {
+    void testBase32BinarySamples() throws Exception {
         final Base32 codec = new Base32();
         for (final Object[] element : BASE32_BINARY_TEST_CASES) {
             final String expected;
@@ -239,7 +242,7 @@ public class Base32Test {
     }
 
     @Test
-    public void testBase32BinarySamplesReverse() throws Exception {
+    void testBase32BinarySamplesReverse() throws Exception {
         final Base32 codec = new Base32();
         for (final Object[] element : BASE32_BINARY_TEST_CASES) {
             assertArrayEquals((byte[]) element[0], codec.decode((String) element[1]));
@@ -247,7 +250,7 @@ public class Base32Test {
     }
 
     @Test
-    public void testBase32Chunked() throws Exception {
+    void testBase32Chunked() throws Exception {
         final Base32 codec = new Base32(20);
         for (final String[] element : BASE32_TEST_CASES_CHUNKED) {
             assertEquals(element[1], codec.encodeAsString(element[0].getBytes(CHARSET_UTF8)));
@@ -255,47 +258,63 @@ public class Base32Test {
     }
 
     @Test
-    public void testBase32DecodingOfTrailing10Bits() {
+    void testBase32DecodingOfTrailing10Bits() {
         assertBase32DecodingOfTrailingBits(10);
     }
 
     @Test
-    public void testBase32DecodingOfTrailing15Bits() {
+    void testBase32DecodingOfTrailing15Bits() {
         assertBase32DecodingOfTrailingBits(15);
     }
 
     @Test
-    public void testBase32DecodingOfTrailing20Bits() {
+    void testBase32DecodingOfTrailing20Bits() {
         assertBase32DecodingOfTrailingBits(20);
     }
 
     @Test
-    public void testBase32DecodingOfTrailing25Bits() {
+    void testBase32DecodingOfTrailing25Bits() {
         assertBase32DecodingOfTrailingBits(25);
     }
 
     @Test
-    public void testBase32DecodingOfTrailing30Bits() {
+    void testBase32DecodingOfTrailing30Bits() {
         assertBase32DecodingOfTrailingBits(30);
     }
 
     @Test
-    public void testBase32DecodingOfTrailing35Bits() {
+    void testBase32DecodingOfTrailing35Bits() {
         assertBase32DecodingOfTrailingBits(35);
     }
 
     @Test
-    public void testBase32DecodingOfTrailing5Bits() {
+    void testBase32DecodingOfTrailing5Bits() {
         assertBase32DecodingOfTrailingBits(5);
     }
 
     @Test
-    public void testBase32HexImpossibleSamples() {
+    void testBase32HexImpossibleSamples() {
         testImpossibleCases(new Base32(0, null, true, BaseNCodec.PAD_DEFAULT, CodecPolicy.STRICT), BASE32HEX_IMPOSSIBLE_CASES);
+        // @formatter:off
+        testImpossibleCases(Base32.builder()
+                .setHexEncodeTable(true)
+                .setDecodingPolicy(CodecPolicy.STRICT)
+                .get(), BASE32HEX_IMPOSSIBLE_CASES);
+        // @formatter:on
+        // overrides, last set wins
+        // @formatter:off
+        testImpossibleCases(Base32.builder()
+                .setHexDecodeTable(false)
+                .setHexDecodeTable(true)
+                .setHexEncodeTable(false)
+                .setHexEncodeTable(true)
+                .setDecodingPolicy(CodecPolicy.STRICT)
+                .get(), BASE32HEX_IMPOSSIBLE_CASES);
+        // @formatter:on
     }
 
     @Test
-    public void testBase32HexSamples() throws Exception {
+    void testBase32HexSamples() throws Exception {
         final Base32 codec = new Base32(true);
         for (final String[] element : BASE32HEX_TEST_CASES) {
             assertEquals(element[1], codec.encodeAsString(element[0].getBytes(CHARSET_UTF8)));
@@ -303,7 +322,7 @@ public class Base32Test {
     }
 
     @Test
-    public void testBase32HexSamplesReverse() throws Exception {
+    void testBase32HexSamplesReverse() throws Exception {
         final Base32 codec = new Base32(true);
         for (final String[] element : BASE32HEX_TEST_CASES) {
             assertEquals(element[0], new String(codec.decode(element[1]), CHARSET_UTF8));
@@ -311,7 +330,7 @@ public class Base32Test {
     }
 
     @Test
-    public void testBase32HexSamplesReverseLowercase() throws Exception {
+    void testBase32HexSamplesReverseLowercase() throws Exception {
         final Base32 codec = new Base32(true);
         for (final String[] element : BASE32HEX_TEST_CASES) {
             assertEquals(element[0], new String(codec.decode(element[1].toLowerCase()), CHARSET_UTF8));
@@ -319,12 +338,12 @@ public class Base32Test {
     }
 
     @Test
-    public void testBase32ImpossibleChunked() {
+    void testBase32ImpossibleChunked() {
         testImpossibleCases(new Base32(20, BaseNCodec.CHUNK_SEPARATOR, false, BaseNCodec.PAD_DEFAULT, CodecPolicy.STRICT), BASE32_IMPOSSIBLE_CASES_CHUNKED);
     }
 
     @Test
-    public void testBase32ImpossibleSamples() {
+    void testBase32ImpossibleSamples() {
         testImpossibleCases(new Base32(0, null, false, BaseNCodec.PAD_DEFAULT, CodecPolicy.STRICT), BASE32_IMPOSSIBLE_CASES);
     }
 
@@ -339,7 +358,7 @@ public class Base32Test {
     }
 
     @Test
-    public void testBase32Samples() throws Exception {
+    void testBase32Samples() throws Exception {
         final Base32 codec = new Base32();
         for (final String[] element : BASE32_TEST_CASES) {
             assertEquals(element[1], codec.encodeAsString(element[0].getBytes(CHARSET_UTF8)));
@@ -347,7 +366,7 @@ public class Base32Test {
     }
 
     @Test
-    public void testBase32SamplesNonDefaultPadding() throws Exception {
+    void testBase32SamplesNonDefaultPadding() throws Exception {
         final Base32 codec = new Base32((byte) 0x25); // '%' <=> 0x25
 
         for (final String[] element : BASE32_PAD_TEST_CASES) {
@@ -356,13 +375,48 @@ public class Base32Test {
     }
 
     @Test
-    public void testCodec200() {
+    void testBuilderCodecPolicy() {
+        assertEquals(CodecPolicy.LENIENT, Base32.builder().get().getCodecPolicy());
+        assertEquals(CodecPolicy.LENIENT, Base32.builder().setDecodingPolicy(CodecPolicy.LENIENT).get().getCodecPolicy());
+        assertEquals(CodecPolicy.STRICT, Base32.builder().setDecodingPolicy(CodecPolicy.STRICT).get().getCodecPolicy());
+        assertEquals(CodecPolicy.LENIENT, Base32.builder().setDecodingPolicy(CodecPolicy.STRICT).setDecodingPolicy(null).get().getCodecPolicy());
+        assertEquals(CodecPolicy.LENIENT, Base32.builder().setDecodingPolicy(null).get().getCodecPolicy());
+    }
+
+    @Test
+    void testBuilderLineAttributes() {
+        assertNull(Base32.builder().get().getLineSeparator());
+        assertNull(Base32.builder().setLineSeparator(BaseNCodec.CHUNK_SEPARATOR).get().getLineSeparator());
+        assertArrayEquals(BaseNCodec.CHUNK_SEPARATOR, Base32.builder().setLineLength(4).setLineSeparator(BaseNCodec.CHUNK_SEPARATOR).get().getLineSeparator());
+        assertArrayEquals(BaseNCodec.CHUNK_SEPARATOR, Base32.builder().setLineLength(4).setLineSeparator(null).get().getLineSeparator());
+        assertArrayEquals(BaseNCodec.CHUNK_SEPARATOR, Base32.builder().setLineLength(10).setLineSeparator(null).get().getLineSeparator());
+        assertNull(Base32.builder().setLineLength(-1).setLineSeparator(null).get().getLineSeparator());
+        assertNull(Base32.builder().setLineLength(0).setLineSeparator(null).get().getLineSeparator());
+        assertArrayEquals(new byte[] { 1 }, Base32.builder().setLineLength(4).setLineSeparator((byte) 1).get().getLineSeparator());
+        assertEquals("MZXXQ===", Base32.builder().setLineLength(4).get().encodeToString("fox".getBytes(CHARSET_UTF8)));
+    }
+
+    @Test
+    void testBuilderPadingByte() {
+        assertNull(Base32.builder().get().getLineSeparator());
+        assertNull(Base32.builder().setLineSeparator(BaseNCodec.CHUNK_SEPARATOR).get().getLineSeparator());
+        assertArrayEquals(BaseNCodec.CHUNK_SEPARATOR, Base32.builder().setLineLength(4).setLineSeparator(BaseNCodec.CHUNK_SEPARATOR).get().getLineSeparator());
+        assertArrayEquals(BaseNCodec.CHUNK_SEPARATOR, Base32.builder().setLineLength(4).setLineSeparator(null).get().getLineSeparator());
+        assertArrayEquals(BaseNCodec.CHUNK_SEPARATOR, Base32.builder().setLineLength(10).setLineSeparator(null).get().getLineSeparator());
+        assertNull(Base32.builder().setLineLength(-1).setLineSeparator(null).get().getLineSeparator());
+        assertNull(Base32.builder().setLineLength(0).setLineSeparator(null).get().getLineSeparator());
+        assertArrayEquals(new byte[] { 1 }, Base32.builder().setLineLength(4).setLineSeparator((byte) 1).get().getLineSeparator());
+        assertEquals("MZXXQ___", Base32.builder().setLineLength(4).setPadding((byte) '_').get().encodeToString("fox".getBytes(CHARSET_UTF8)));
+    }
+
+    @Test
+    void testCodec200() {
         final Base32 codec = new Base32(true, (byte) 'W'); // should be allowed
         assertNotNull(codec);
     }
 
     @Test
-    public void testConstructors() {
+    void testConstructors() {
         Base32 base32;
         base32 = new Base32();
         base32 = new Base32(-1);
@@ -373,13 +427,12 @@ public class Base32Test {
         // even when line length is negative.
         base32 = new Base32(-1, new byte[] { 'A' });
         base32 = new Base32(32, new byte[] { '$' }); // OK
-        assertThrows(IllegalArgumentException.class, () -> new Base32(32, null), "null line separator");
+        assertArrayEquals(new byte[0], new Base32(32, null).getLineSeparator(), "null line separator use an empty array");
         assertThrows(IllegalArgumentException.class, () -> new Base32(32, new byte[] { 'A' }), "'A' as a line separator");
         assertThrows(IllegalArgumentException.class, () -> new Base32(32, new byte[] { '=' }), "'=' as a line separator");
         assertThrows(IllegalArgumentException.class, () -> new Base32(32, new byte[] { 'A', '$' }), "'A$' as a line separator");
         assertThrows(IllegalArgumentException.class, () -> new Base32(32, new byte[] { '\n' }, false, (byte) 'A'), "'A' as padding");
         assertThrows(IllegalArgumentException.class, () -> new Base32(32, new byte[] { '\n' }, false, (byte) ' '), "' ' as padding");
-
         base32 = new Base32(32, new byte[] { ' ', '$', '\n', '\r', '\t' }); // OK
         assertNotNull(base32);
     }
@@ -388,7 +441,7 @@ public class Base32Test {
      * Test encode and decode of empty byte array.
      */
     @Test
-    public void testEmptyBase32() {
+    void testEmptyBase32() {
         byte[] empty = {};
         byte[] result = new Base32().encode(empty);
         assertEquals(0, result.length, "empty Base32 encode");
@@ -410,7 +463,7 @@ public class Base32Test {
     }
 
     @Test
-    public void testIsInAlphabet() {
+    void testIsInAlphabet() {
         // invalid bounds
         Base32 b32 = new Base32(true);
         assertFalse(b32.isInAlphabet((byte) 0));
@@ -441,7 +494,7 @@ public class Base32Test {
         // hex table
         b32 = new Base32(true);
         for (char c = '0'; c <= '9'; c++) {
-            assertTrue(b32.isInAlphabet((byte) c));
+            assertTrue(b32.isInAlphabet((byte) c), String.valueOf(c));
         }
         for (char c = 'A'; c <= 'V'; c++) {
             assertTrue(b32.isInAlphabet((byte) c));
@@ -458,7 +511,7 @@ public class Base32Test {
     }
 
     @Test
-    public void testRandomBytes() {
+    void testRandomBytes() {
         for (int i = 0; i < 20; i++) {
             final Base32 codec = new Base32();
             final byte[][] b = BaseNTestData.randomData(codec, i);
@@ -468,7 +521,7 @@ public class Base32Test {
     }
 
     @Test
-    public void testRandomBytesChunked() {
+    void testRandomBytesChunked() {
         for (int i = 0; i < 20; i++) {
             final Base32 codec = new Base32(10);
             final byte[][] b = BaseNTestData.randomData(codec, i);
@@ -478,7 +531,7 @@ public class Base32Test {
     }
 
     @Test
-    public void testRandomBytesHex() {
+    void testRandomBytesHex() {
         for (int i = 0; i < 20; i++) {
             final Base32 codec = new Base32(true);
             final byte[][] b = BaseNTestData.randomData(codec, i);
@@ -488,7 +541,7 @@ public class Base32Test {
     }
 
     @Test
-    public void testSingleCharEncoding() {
+    void testSingleCharEncoding() {
         for (int i = 0; i < 20; i++) {
             Base32 codec = new Base32();
             final BaseNCodec.Context context = new BaseNCodec.Context();
